@@ -1,3 +1,4 @@
+use std::borrow::Cow;
 use std::str;
 use std::sync::Arc;
 use std::sync::atomic::AtomicUsize;
@@ -24,9 +25,20 @@ pub use certd::CertD;
 pub mod certs;
 pub use certs::Certs;
 
+// The keyserver backend is optional.
+#[cfg(feature = "keyserver")]
 pub mod keyserver;
+#[cfg(feature = "keyserver")]
 pub use keyserver::KeyServer;
 
+// If it is disabled, we swap in a dummy to reduce changes to the rest
+// of the code.
+pub mod no_keyserver;
+#[cfg(not(feature = "keyserver"))]
+use no_keyserver as keyserver;
+#[cfg(not(feature = "keyserver"))]
+pub(crate) use keyserver::KeyServer;
+
 pub mod pep;
 pub use pep::Pep;
 
@@ -174,24 +186,17 @@ impl UserIDQueryParams {
                 // Substring search.
                 let mut userid = if *email {
                     if let Ok(Some(email)) = userid.email_normalized() {
-                        email
+                        Cow::Owned(email)
                     } else {
                         t!("User ID does not contain a valid email address");
                         return false;
                     }
                 } else {
-                    if let Ok(userid)
-                        = String::from_utf8(userid.value().to_vec())
-                    {
-                        userid
-                    } else {
-                        t!("User ID is not UTF-8 encoded");
-                        return false;
-                    }
+                    String::from_utf8_lossy(userid.value())
                 };
 
                 if *ignore_case {
-                    userid = userid.to_lowercase();
+                    userid = Cow::Owned(userid.to_lowercase());
                 }
 
                 let mut pattern = pattern;
@@ -581,12 +586,30 @@ pub trait Store<'a> {
 
     /// Returns certificates that have a User ID matching the
     /// specified pattern according to the query parameters.
+    ///
+    /// User IDs are interpreted as UTF-8 strings.  If a user ID is
+    /// not valid UTF-8, then it will be converted to UTF-8 in a lossy
+    /// manner using the same semantics as
+    /// [`String::from_utf8_lossy`].
+    ///
+    /// This function returns all matching user IDs, even those that
+    /// are not self signed.  It's up to the caller to authenticate
+    /// the returned user IDs.
     fn select_userid(&self, query: &UserIDQueryParams, pattern: &str)
         -> Result<Vec<Arc<LazyCert<'a>>>>;
 
     /// Performs an exact match on the User ID.
     ///
     /// The pattern is anchored, and the match is case sensitive.
+    ///
+    /// The user ID is interpreted as a UTF-8 string.  If the user ID
+    /// is not valid UTF-8, then it will be converted to UTF-8 in a
+    /// lossy manner using the same semantics as
+    /// [`String::from_utf8_lossy`].
+    ///
+    /// This function returns all matching user IDs, even those that
+    /// are not self signed.  It's up to the caller to authenticate
+    /// the returned user IDs.
     fn lookup_by_userid(&self, userid: &UserID) -> Result<Vec<Arc<LazyCert<'a>>>> {
         self.select_userid(
             &UserIDQueryParams::new()
@@ -594,13 +617,22 @@ pub trait Store<'a> {
                 .set_anchor_start(true)
                 .set_anchor_end(true)
                 .set_ignore_case(false),
-            &String::from_utf8(userid.value().to_vec())?)
+            &String::from_utf8_lossy(userid.value()))
     }
 
     /// Performs a case insenitive, substring match on the User ID.
     ///
     /// The pattern is not anchored, and it is matched case
     /// insensitively.
+    ///
+    /// User IDs are interpreted as UTF-8 strings.  If a user ID is
+    /// not valid UTF-8, then it will be converted to UTF-8 in a lossy
+    /// manner using the same semantics as
+    /// [`String::from_utf8_lossy`].
+    ///
+    /// This function returns all matching user IDs, even those that
+    /// are not self signed.  It's up to the caller to authenticate
+    /// the returned user IDs.
     fn grep_userid(&self, pattern: &str) -> Result<Vec<Arc<LazyCert<'a>>>> {
         self.select_userid(
             &UserIDQueryParams::new()
@@ -617,6 +649,15 @@ pub trait Store<'a> {
     /// The pattern is interpreted as an email address.  It is first
     /// normalized, and then matched against the normalized email
     /// address, it is anchored, and the match is case sensitive.
+    ///
+    /// User IDs are interpreted as UTF-8 strings.  If a user ID is
+    /// not valid UTF-8, then it will be converted to UTF-8 in a lossy
+    /// manner using the same semantics as
+    /// [`String::from_utf8_lossy`].
+    ///
+    /// This function returns all matching user IDs, even those that
+    /// are not self signed.  It's up to the caller to authenticate
+    /// the returned user IDs.
     fn lookup_by_email(&self, email: &str) -> Result<Vec<Arc<LazyCert<'a>>>> {
         let userid = crate::email_to_userid(&email)?;
         let email = userid.email_normalized()?.expect("have one");
@@ -636,6 +677,15 @@ pub trait Store<'a> {
     /// The pattern is matched against the normalized email address,
     /// it is not anchored, and it is matched case insensitively.  The
     /// pattern itself is *not* normalized.
+    ///
+    /// User IDs are interpreted as UTF-8 strings.  If a user ID is
+    /// not valid UTF-8, then it will be converted to UTF-8 in a lossy
+    /// manner using the same semantics as
+    /// [`String::from_utf8_lossy`].
+    ///
+    /// This function returns all matching user IDs, even those that
+    /// are not self signed.  It's up to the caller to authenticate
+    /// the returned user IDs.
     fn grep_email(&self, pattern: &str) -> Result<Vec<Arc<LazyCert<'a>>>> {
         self.select_userid(
             &UserIDQueryParams::new()
@@ -657,6 +707,15 @@ pub trait Store<'a> {
     /// not start with an `@`.  This does not match subdomains.  That
     /// is, it will match `alice@foo.bar.com` when searching for
     /// `bar.com`.
+    ///
+    /// User IDs are interpreted as UTF-8 strings.  If a user ID is
+    /// not valid UTF-8, then it will be converted to UTF-8 in a lossy
+    /// manner using the same semantics as
+    /// [`String::from_utf8_lossy`].
+    ///
+    /// This function returns all matching user IDs, even those that
+    /// are not self signed.  It's up to the caller to authenticate
+    /// the returned user IDs.
     fn lookup_by_email_domain(&self, domain: &str) -> Result<Vec<Arc<LazyCert<'a>>>> {
         let localpart = "localpart";
         let email = format!("{}@{}", localpart, domain);
