@@ -501,7 +501,7 @@ CREATE TABLE IF NOT EXISTS entries (
 
         // Add the list of entries that need to be updated to an
         // in-memory table, and then do a single insert.  Using an
-        // INSERT per update takes about five times as longer.
+        // INSERT per update takes about five times as long.
         tx.execute("ATTACH ':memory:' as in_memory", ())?;
 
         tx.execute("CREATE TABLE in_memory.accessed \
@@ -543,10 +543,8 @@ CREATE TABLE IF NOT EXISTS entries (
 
         if inserted > 0 || accessed_old > 0 {
             tx.execute(
-                "INSERT INTO entries \
-                 SELECT value, ?1 FROM in_memory.accessed \
-                 WHERE true \
-                 ON CONFLICT DO UPDATE SET last_accessed = ?1",
+                "INSERT OR REPLACE INTO entries \
+                 SELECT value, ?1 FROM in_memory.accessed",
                 params!(now))?;
         }
 
